import os
from pathlib import Path
import tomlkit
from tomlkit.toml_document import TOMLDocument

def read(pyprojectPath: Path) -> TOMLDocument:
    with pyprojectPath.open('r', encoding='utf-8') as t:
        return tomlkit.parse(t.read())

def getPath(workingDir=os.getcwd()):
    return Path(workingDir).joinpath('pyproject.toml')



import sys

if sys.version_info >= (3, 8):
    from importlib import metadata as importlib_metadata # pylint: disable = no-name-in-module
else:
    import importlib_metadata # pylint: disable = import-error

def getByKey(key: str):
    return importlib_metadata.entry_points().get(key, ())



from pathlib import Path
from typing import List
from box import Box
from injecta.compiler.CompilerPassInterface import CompilerPassInterface
from injecta.config.ConfigLoader import ConfigLoader
from injecta.config.ConfigMerger import ConfigMerger
from injecta.service.Service import Service
from injecta.package.pathResolver import resolvePath
from injecta.service.ServiceAlias import ServiceAlias
from pyfonybundles.Bundle import Bundle

class BundleManager:

    def __init__(self, bundles: List[Bundle]):
        self.__bundles = bundles
        self.__configLoader = ConfigLoader()
        self.__configMerger = ConfigMerger()

    def getCompilerPasses(self) -> List[CompilerPassInterface]:
        compilerPasses = []

        for bundle in self.__bundles:
            compilerPasses += bundle.getCompilerPasses()

        return compilerPasses

    def getBundlesConfig(self) -> dict:
        config = dict()

        for bundle in self.__bundles:
            rootModuleName = bundle.__module__[:bundle.__module__.find('.')]
            configBasePath = resolvePath(rootModuleName) + '/_config'

            for configFileName in bundle.getConfigFiles():
                configFilePath = Path(configBasePath + '/' + configFileName)
                newConfig = self.__configLoader.load(configFilePath)

                config = self.__configMerger.merge(config, newConfig, False)

        return config

    def getProjectBundlesConfig(self, bundlesConfigsDir: str) -> dict:
        config = dict()

        for bundle in self.__bundles:
            rootPackageName = bundle.__module__[:bundle.__module__.find('.')]
            projectBundleConfigPath = Path(bundlesConfigsDir + '/' + rootPackageName + '.yaml')

            if projectBundleConfigPath.exists():
                projectBundleConfig = self.__configLoader.load(projectBundleConfigPath)

                config = self.__configMerger.merge(config, projectBundleConfig)

        return config

    def modifyRawConfig(self, rawConfig: dict) -> dict:
        for bundle in self.__bundles:
            rawConfig = bundle.modifyRawConfig(rawConfig)

        return rawConfig

    def modifyServices(self, services: List[Service], aliases: List[ServiceAlias], parameters: Box):
        for bundle in self.__bundles:
            services, aliases = bundle.modifyServices(services, aliases, parameters)

        return services, aliases

    def modifyParameters(self, parameters: Box) -> Box:
        for bundle in self.__bundles:
            parameters = bundle.modifyParameters(parameters)

        return parameters



from typing import List
from box import Box
from injecta.compiler.CompilerPassInterface import CompilerPassInterface
from injecta.container.ContainerInterface import ContainerInterface
from injecta.service.Service import Service
from injecta.service.ServiceAlias import ServiceAlias

class Bundle:

    def getConfigFiles(self):
        return ['config.yaml']

    def getCompilerPasses(self) -> List[CompilerPassInterface]:
        return []

    def modifyRawConfig(self, rawConfig: dict) -> dict:
        return rawConfig

    def modifyServices(self, services: List[Service], aliases: List[ServiceAlias], parameters: Box): # pylint: disable = unused-argument
        return services, aliases

    def modifyParameters(self, parameters: Box) -> Box:
        return parameters

    def boot(self, container: ContainerInterface):
        pass



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['pyfonybundles', 'pyfonybundles.loader']

package_data = \
{'': ['*']}

install_requires = \
['importlib-metadata>=3.3.0',
 'injecta>=0.9.0a4',
 'python-box>=3.4.0,<3.5.0',
 'tomlkit>=0.5.0,<0.6.0']

setup_kwargs = {
    'name': 'pyfony-bundles',
    'version': '0.2.5a7',
    'description': 'Pyfony Framework bundles base package',
    'long_description': 'Pyfony Framework bundles base package',
    'author': 'Jiri Koutny',
    'author_email': 'jiri.koutny@datasentics.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pyfony/pyfony-bundles',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.3,<3.8.0',
}


setup(**setup_kwargs)



from typing import List
from injecta.dtype.classLoader import loadClass
from pyfonybundles.Bundle import Bundle
from pyfonybundles.loader import entryPointsReader, pyprojectReader

def getEntryPoints():
    entryPoints = entryPointsReader.getByKey('pyfony.bundle')

    for entryPoint in entryPoints:
        _checkName(entryPoint.name, entryPoint.value)

    return entryPoints

def loadBundles() -> List[Bundle]:
    return [entryPoint.load()() for entryPoint in getEntryPoints()]

def loadBundlesWithCurrent() -> List[Bundle]:
    bundles = loadBundles()

    rawConfig = pyprojectReader.read(pyprojectReader.getPath())

    if not _entryPointDefined(rawConfig):
        raise Exception('Missing entry point [tool.poetry.plugins."pyfony.bundle"] in pyproject.toml')

    bundle = _loadDirectly(rawConfig)()
    bundles.append(bundle)

    return bundles

def _entryPointDefined(rawConfig):
    return (
        'tool' in rawConfig
        and 'poetry' in rawConfig['tool']
        and 'plugins' in rawConfig['tool']['poetry']
        and 'pyfony.bundle' in rawConfig['tool']['poetry']['plugins']
    )

def _loadDirectly(rawConfig):
    entryPoints = rawConfig['tool']['poetry']['plugins']['pyfony.bundle']

    for name, val in entryPoints.items():
        _checkName(name, val)

    return _parse(entryPoints['create'])

def _checkName(name: str, value):
    if name != 'create':
        raise Exception(f'Unexpected entry point name "{name}" for {value}')

def _parse(val):
    moduleName, classAndMethod = val.split(':')

    if '.' not in classAndMethod:
        return loadClass(moduleName, classAndMethod)

    className, functionName = classAndMethod.split('.')

    class_ = loadClass(moduleName, className) # pylint: disable = invalid-name

    return getattr(class_, functionName)
